'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ClientError = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _retry = require('retry');

var _retry2 = _interopRequireDefault(_retry);

var _es6Error = require('es6-error');

var _es6Error2 = _interopRequireDefault(_es6Error);

var _rateLimit = require('../rate-limit');

var _rateLimit2 = _interopRequireDefault(_rateLimit);

var _package = require('../../package.json');

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = require('debug')('graphbrainz:api/client');

// If the `request` callback returns an error, it indicates a failure at a lower
// level than the HTTP response itself. If it's any of the following error
// codes, we should retry.
const RETRY_CODES = {
  ECONNRESET: true,
  ENOTFOUND: true,
  ESOCKETTIMEDOUT: true,
  ETIMEDOUT: true,
  ECONNREFUSED: true,
  EHOSTUNREACH: true,
  EPIPE: true,
  EAI_AGAIN: true
};

class ClientError extends _es6Error2.default {
  constructor(message, statusCode) {
    super(message);
    this.statusCode = statusCode;
  }
}

exports.ClientError = ClientError;
class Client {
  constructor({
    baseURL,
    userAgent = `${_package2.default.name}/${_package2.default.version} ` + `( ${_package2.default.homepage || _package2.default.author.url || _package2.default.author.email} )`,
    extraHeaders = {},
    errorClass = ClientError,
    timeout = 60000,
    limit = 1,
    period = 1000,
    concurrency = 10,
    retries = 10,
    // It's OK for `retryDelayMin` to be less than one second, even 0, because
    // `RateLimit` will already make sure we don't exceed the API rate limit.
    // We're not doing exponential backoff because it will help with being
    // rate limited, but rather to be chill in case MusicBrainz is returning
    // some other error or our network is failing.
    retryDelayMin = 100,
    retryDelayMax = 60000,
    randomizeRetry = true
  } = {}) {
    this.baseURL = baseURL;
    this.userAgent = userAgent;
    this.extraHeaders = extraHeaders;
    this.errorClass = errorClass;
    this.timeout = timeout;
    this.limiter = new _rateLimit2.default({ limit, period, concurrency });
    this.retryOptions = {
      retries,
      minTimeout: retryDelayMin,
      maxTimeout: retryDelayMax,
      randomize: randomizeRetry
    };
  }

  /**
   * Determine if we should retry the request based on the given error.
   * Retry any 5XX response from MusicBrainz, as well as any error in
   * `RETRY_CODES`.
   */
  shouldRetry(err) {
    if (err instanceof this.errorClass) {
      return err.statusCode >= 500 && err.statusCode < 600;
    }
    return RETRY_CODES[err.code] || false;
  }

  parseErrorMessage(response, body) {
    return typeof body === 'string' && body ? body : `${response.statusCode}`;
  }

  /**
   * Send a request without any retrying or rate limiting.
   * Use `get` instead.
   */
  _get(path, options = {}, info = {}) {
    return new _promise2.default((resolve, reject) => {
      options = {
        baseUrl: this.baseURL,
        url: path,
        gzip: true,
        timeout: this.timeout,
        ...options,
        headers: {
          'User-Agent': this.userAgent,
          ...this.extraHeaders,
          ...options.headers
        }
      };

      const url = `${options.baseUrl}${options.url}`;

      debug(`Sending request. url=${url} attempt=${info.currentAttempt}`);

      (0, _request2.default)(options, (err, response, body) => {
        if (err) {
          debug(`Error: “${err}” url=${url}`);
          reject(err);
        } else if (response.statusCode >= 400) {
          const message = this.parseErrorMessage(response, body);
          debug(`Error: “${message}” url=${url}`);
          const ClientError = this.errorClass;
          reject(new ClientError(message, response.statusCode));
        } else if (options.method === 'HEAD') {
          resolve(response.headers);
        } else {
          resolve(body);
        }
      });
    });
  }

  /**
   * Send a request with retrying and rate limiting.
   */
  get(path, options = {}) {
    return new _promise2.default((resolve, reject) => {
      const fn = this._get.bind(this);
      const operation = _retry2.default.operation(this.retryOptions);
      operation.attempt(currentAttempt => {
        // This will increase the priority in our `RateLimit` queue for each
        // retry, so that newer requests don't delay this one further.
        const priority = currentAttempt;
        this.limiter.enqueue(fn, [path, options, { currentAttempt }], priority).then(resolve).catch(err => {
          if (!this.shouldRetry(err) || !operation.retry(err)) {
            reject(operation.mainError() || err);
          }
        });
      });
    });
  }
}
exports.default = Client;